/**
 * This method will fetch the available manual versions.
 *
 * @returns {Promise} Returns a promise object which contains the available manual versions.
 */
export const getAvailableManualVersions = () => {
    return fetch('versions.php').then(response => response.json());
};

/**
 * Returns the an available manual version.
 *
 * If shop version 3.9.2.3 was provided but no manual version for 3.9.2.3 was found,
 * this method will search the next smaller version which has a manual. If a manual for version
 * 3.9.2.2 would be there, this method would return that as the available version.
 *
 * @param providedVersion Version provided by the GET parameter 'v'
 *
 * @returns {Promise}
 */
export const getApplicableVersion = providedVersion => {
    return getAvailableManualVersions().then(availableManualVersions => {
        
        // Development mode
        if (availableManualVersions[0] === '0.0.0.0') {
            return Promise.resolve();
        }
        
        if (!providedVersion || smallerThanMinVersion(providedVersion) || !startsWithNumberAndDot(providedVersion)) {
            // return latest manual version
            return availableManualVersions[availableManualVersions.length - 1];
        }
        
        // throw away everything after _, eg. 3.9.2.2_beta1 -> 3.9.2.2
        providedVersion = providedVersion.replace(/_(.)*/g, '');
        
        // Provided version is available
        if (availableManualVersions.indexOf(providedVersion) > -1) {
            return providedVersion;
        }
        
        let nextSmallerVersion = providedVersion;
        let found              = false;
        
        // Find the next smaller available manual version.
        while (found === false) {
            const version    = nextSmallerVersion.split('.');
            const maxVersion = 15;
            
            if (version[3] > 0) {
                version[3] = Number(version[3]) - 1;
            } else if (version[2] > 0) {
                version[2] = Number(version[2]) - 1;
                version[3] = Number(version[3]) || maxVersion;
            } else if (version[1] > 0) {
                version[1] = Number(version[1]) - 1;
                version[2] = Number(version[2]) || maxVersion;
                version[3] = Number(version[3]) || maxVersion;
            } else if (version[0] > 0) {
                version[0] = Number(version[0]) - 1;
                version[1] = Number(version[1]) || maxVersion;
                version[2] = Number(version[2]) || maxVersion;
                version[3] = Number(version[3]) || maxVersion;
            }
            
            nextSmallerVersion = version.join('.');
            
            // Alternative version could be found
            if (availableManualVersions.indexOf(nextSmallerVersion) > -1) {
                found = true;
                return nextSmallerVersion;
            }
            
            // Nothing could be found, return the fallback (latest manual)
            if (nextSmallerVersion === '0.0.0.0' || Number(version[0]) <= 0) {
                return availableManualVersions[availableManualVersions.length - 1];
            }
        }
    });
};

/**
 * Checks if the provided version is too old.
 *
 * @param providedVersion Version to check.
 *
 * @returns {boolean}
 */
const smallerThanMinVersion = providedVersion => {
    const smallestManualVersion = 3922;
    const version               = providedVersion.replace(/\./g, '').replace(/_[A-Za-z]*\d/, '');
    return Number(version) < smallestManualVersion;
};


/**
 * Returns whether the provided string starts with a number and a dot or not.
 *
 * @param string String to check.
 *
 * @returns {boolean} True, if the provided string starts with a number and a dot.
 */
const startsWithNumberAndDot = (string) => {
    return !!string.match(/^[\d]+\./);
}