/* --------------------------------------------------------------
 ManualDocument.js 2016-09-16
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2016 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

/**
 * Manual Document Class
 *
 * This class handles the manipulation of the generated manual HTML document and provides an easy
 * way to fetch the table of contents and the manual content separately.
 */
class ManualDocument {
	/**
	 * Class Constructor
	 *
	 * @param {String} html Manual Document HTML
	 * @param {String} version Selected Manual Version
	 * @param {boolean} isStandalone Whether the manual document is a single section
	 */
	constructor(html, version, isStandalone = false) {
		/**
		 * @type {HTMLElement}
		 */
		this.manualDocument = this._normalize(html, version);
		/**
		 * @type {boolean}
		 */
		this.isStandalone = isStandalone;
		/**
		 * @type {String}
		 */
		this.version = version;
	}

	/**
	 * Get Page Title
	 *
	 * @return {String} Returns only the title text.
	 */
	getPageTitle() {
		return this.manualDocument.querySelector('.document-title').innerHTML;
	}

	/**
	 * Get the table of contents HTML.
	 *
	 * @return {String} Returns the table of contents markup.
	 */
	getTableOfContents() {
		return this.manualDocument.querySelector('.toc-container').outerHTML;
	}

	/**
	 * Get the manual contents HTML.
	 *
	 * @return {String} Returns the title and topics markup.
	 */
	getManualContents() {
		const topics = this.manualDocument.querySelectorAll('.topic');
		let html = '';

		if (!this.isStandalone) {
			html += `<h1>${this.getPageTitle()}</h1>`;
		}

		topics.forEach(topic => {
			html += topic.outerHTML;
		});

		return html;
	}

	/**
	 * Get the HTML of a specific section in the manual
	 * @param sectionId the section identifier to get content for.
	 * @returns {String} the content's markup
	 */
	getManualSection(sectionId) {
		const topic = this.manualDocument.querySelector('#' + sectionId);
		return topic;
	}

	/**
	 * Normalize Manual HTML
	 *
	 * This method will make the required modifications to the provided manual HTML. Modifications are required
	 * because the original HTML contains elements that are not required. In addition links and paths need to be
	 * adjusted because they will be served within a different page from the one they were generated for.
	 *
	 * @param {String} html Manual HTML to be normalized.
	 * @param {String} version Selected display version of the manual.
	 *
	 * @return {HTMLElement} Returns an HTML element which contains the normalized markup.
	 *
	 * @private
	 */
	_normalize(html, version) {
		const manualDocument = document.createElement('manualDocument');
		manualDocument.innerHTML = html;

		const manualBaseUrl = version ? `docs/${version}/manual/` : `docs/manual/`;

		// Normalize image paths.
		manualDocument.querySelectorAll('img').forEach((image) => {
		    const fileExtensionRegex = /\.\w+$/;
			let src = image.getAttribute('src');
			
			// In some cases, file extensions were written in UPPERCASE which prevented the
            // image from being loaded, since file extensions are written in lowercase.
			const fileExtension = src.match(fileExtensionRegex).toString();
		    src = src.replace(fileExtensionRegex, fileExtension.toLowerCase());

			if ((src && src.indexOf('images') === 0) || src.indexOf('resources') === 0) {
				image.dataset.src = manualBaseUrl + src;
				image.classList.add('img-responsive');
                image.classList.add('lazy');
                image.setAttribute('src', ''); // attribute will be set by lazy loading
			}
		});

		// Hide existing navigation icons.
		manualDocument.querySelectorAll('.topic-navigation-bar').forEach(topicNavigationBar => {
			// topicNavigationBar.style.display = 'none';
			topicNavigationBar.remove();
		});

		// Remove border attribute from tables.
		manualDocument.querySelectorAll('table[border="1"]').forEach(table => {
			table.removeAttribute('border');
			table.classList.add('table', 'table-bordered');
		});

		manualDocument.querySelectorAll('tr.row').forEach(tr => {
			tr.classList.remove('row');
		});

		return manualDocument;
	}
}

export default ManualDocument;
