/* --------------------------------------------------------------
 ManualSearchBox.js 2016-09-22
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2016 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

/**
 * @type {Number}
 */
const ENTER_KEY_CODE = 13;

/**
 * Mark Search Occurrences
 *
 * This method will search recursively every Text node for the desired keyword and mark it with a wrapper. This
 * is done because it's not possible to replace the occurrence with HTML directly.
 *
 * @param {HTMLElement} container Container to be searched.
 * @param {String} keyword Search keyword to be found.
 * @param {String} [wrapper = '{{{$1}}}'] Provide a wrapper string including "$1" at the place of the occurrence.
 *
 * @private
 */
function _markSearchOccurrences(container, keyword, wrapper = '{{{$1}}}') {
	Array.from(container.childNodes).forEach(node => {
		if (node.children) {
			_markSearchOccurrences(node, keyword);
		} else if (keyword) {
			node.textContent = node.textContent.replace(new RegExp(`(${keyword})`, 'ig'), wrapper);
		}
	});
}

/**
 * Display a message.
 *
 * @param {String} message The message to be displayed.
 *
 * @param {String} type Message type eg. info | warning. Default: info
 */
function _display(message, type = 'info') {
    const manualContents = document.querySelector('#manual');
    const div = document.createElement('div');
    
    div.innerHTML = `
        <div class="${type} message-to-user">
            ${message}
            <i class="fa fa-times pull-right remove" aria-hidden="true"></i>
        </div>
    `;
    
    manualContents.prepend(div);
    
    const removeButton = document.querySelector('i.remove');
    removeButton.addEventListener('click', () => div.remove());
}

/**
 * Next Button Click Event Handler
 *
 * @param {HTMLElement} manualContents Manual contents container.
 * @param {PerfectScrollbar} manualContentsScrollbar PerfectScrollbar instance.
 *
 * @private
 */
function _onNextButtonClick(manualContents, manualContentsScrollbar) {
	const highlights = manualContents.querySelectorAll('.highlight');

	Array.from(highlights).every((highlight, index) => {
		if (highlight.classList.contains('current')) {
			const newCurrentHighlight =
				index === highlights.length - 1 ? highlights[0] : highlights[index + 1];
			newCurrentHighlight.classList.add('current');

			if (manualContents.classList.contains('ps')) {
				manualContents.scrollTop = newCurrentHighlight.offsetTop;
				manualContentsScrollbar.update();
			} else {
				const scrollTop =
					newCurrentHighlight.getBoundingClientRect().top + document.body.scrollTop;
				window.scrollTo(0, scrollTop);
			}

			highlight.classList.remove('current');
			return false;
		}
		return true;
	});
}

/**
 * Previous Button Click Handler
 *
 * @param {HTMLElement} manualContents Manual contents container.
 * @param {PerfectScrollbar} manualContentsScrollbar PerfectScrollbar instance.
 *
 * @private
 */
function _onPreviousButtonClick(manualContents, manualContentsScrollbar) {
	const highlights = manualContents.querySelectorAll('.highlight');

	Array.from(highlights).every((highlight, index) => {
		if (highlight.classList.contains('current')) {
			const newCurrentHighlight =
				index === 0 ? highlights[highlights.length - 1] : highlights[index - 1];
			newCurrentHighlight.classList.add('current');

			if (manualContents.classList.contains('ps')) {
				manualContents.scrollTop = newCurrentHighlight.offsetTop;
				manualContentsScrollbar.update();
			} else {
				const scrollTop =
					newCurrentHighlight.getBoundingClientRect().top + document.body.scrollTop;
				window.scrollTo(0, scrollTop);
			}

			highlight.classList.remove('current');
			return false;
		}
		return true;
	});
}

/**
 * Clear Button Click Handler
 *
 * @param {Event} event Click event object.
 * @param {HTMLElement} tableOfContents Table of contents container.
 * @param {HTMLElement} manualContents Manual contents container.
 * @param {PerfectScrollbar} manualContentsScrollbar PerfectScrollbar instance.
 *
 * @private
 */
function _onClearButtonClick(event, tableOfContents, manualContents, manualContentsScrollbar) {
	const inputField = event.currentTarget.parentNode.querySelector('input');

	inputField.value = '';

	const eventData = {
		keyCode: ENTER_KEY_CODE,
		target: inputField
	};

	_onInputFieldKeyUp(eventData, tableOfContents, manualContents, manualContentsScrollbar);
}

/**
 * Input Field Key Up Event Handler
 *
 * @param {Event} event Key up event object.
 * @param {HTMLElement} tableOfContents Table of contents container.
 * @param {HTMLElement} manualContents Manual contents container.
 * @param {PerfectScrollbar} manualContentsScrollbar PerfectScrollbar instance.
 *
 * @private
 */
function _onInputFieldKeyUp(event, tableOfContents, manualContents, manualContentsScrollbar) {
	if (event.keyCode !== ENTER_KEY_CODE) {
		return;
	}
	
	const shownMessage = document.querySelector('.message-to-user');
	shownMessage && shownMessage.remove();
    
    // Escape search keyword.
    const keyword = event.target.value.replace(/[&<>]/g, function(tag) {
        const tagsToReplace = {
            '&': '&amp;',
            '<': '&lt;',
            '>': '&gt;'
        };
        
        return tagsToReplace[tag] || tag;
    });

	// Reset content to the original state.
	manualContents.innerHTML = manualContents.innerHTML.replace(
		new RegExp(`<span class="highlight.*">(.*)</span>`, 'ig'),
		'$1'
	);

	// Will contain found matches and hidden non-matches
	const matches = [];
	const hiddenTableOfContents = [];
	const hiddenTopics = [];
	
	// Perform the keyword search again.
	manualContents.querySelectorAll('.topic').forEach(topic => {
		if (!topic.textContent.match(new RegExp(keyword, 'ig'))) {
			const noMatch = tableOfContents.querySelector(`a[href="#${topic.getAttribute('id')}"]`);
			
			// Hide none matches
			noMatch.parentNode.style.display = 'none';
			topic.style.display = 'none';
			
			// save hidden topics, so that they can be shown again if no matches were found at all
			hiddenTableOfContents.push(noMatch);
			hiddenTopics.push(topic);
			
			return;
		}

		const match = tableOfContents.querySelector(`a[href="#${topic.getAttribute('id')}"]`);
        match.parentNode.style.display = 'block';
		topic.style.display = 'block';
		
        matches.push(match);

		_markSearchOccurrences(topic, keyword);

		// Reset the active toc entries and scroll content to first highlight.
		const activeSectionLink = tableOfContents.querySelector('a.active');

		if (activeSectionLink) {
			activeSectionLink.classList.remove('active');
		}

		manualContents.scrollTop = 0;
		manualContentsScrollbar.update();
	});

	// Highlight marked keyword occurrences.
	manualContents.innerHTML = manualContents.innerHTML.replace(
		new RegExp(`{{{(${keyword})}}}`, 'ig'),
		'<span class="highlight">$1</span>'
	);

	const currentHighlight = manualContents.querySelector('.highlight:first-child');

	if (currentHighlight) {
		currentHighlight.classList.add('current');

		if (manualContents.classList.contains('ps')) {
			manualContents.scrollTop = currentHighlight.offsetTop;
			manualContentsScrollbar.update();
		} else {
			const scrollTop =
				currentHighlight.getBoundingClientRect().top + document.body.scrollTop;
			window.scrollTo(0, scrollTop);
		}
	}
 
	// Show message and reload the page if no matches could be found
	if (!matches.length) {
	    _display('Die Suche für <strong>' + keyword + ' </strong>ergab keine Treffer', 'warning');
	    hiddenTableOfContents.forEach(entry => entry.parentNode.style.display = 'block');
	    hiddenTopics.forEach(topic => topic.style.display = 'block');
    }
}


/**
 * Create Manual Search Box
 *
 * This component will create a custom search box which will look into the manual contents and filter the table
 * of contents accordingly.
 *
 * @param {HTMLElement} tableOfContents The table of contents element.
 * @param {HTMLElement} manualContents The manual contents element.
 * @param {PerfectScrollbar} manualContentsScrollbar PerfectScrollbar instance.
 */
export function createManualSearchBox(tableOfContents, manualContents, manualContentsScrollbar) {
	const inputField = document.createElement('input');
	inputField.classList.add('form-control', 'input-sm');
	inputField.setAttribute('type', 'text');
	inputField.setAttribute('placeholder', 'Suche');
	inputField.addEventListener('keyup', event =>
		_onInputFieldKeyUp(event, tableOfContents, manualContents, manualContentsScrollbar)
	);

	const previousButton = document.createElement('button');
	previousButton.classList.add('btn', 'btn-default', 'btn-sm', 'next');
	previousButton.innerHTML = '<i class="fa fa-arrow-up"></i>';
	previousButton.addEventListener('click', () =>
		_onPreviousButtonClick(manualContents, manualContentsScrollbar)
	);

	const nextButton = document.createElement('button');
	nextButton.classList.add('btn', 'btn-default', 'btn-sm', 'next');
	nextButton.innerHTML = '<i class="fa fa-arrow-down"></i>';
	nextButton.addEventListener('click', () =>
		_onNextButtonClick(manualContents, manualContentsScrollbar)
	);

	const clearButton = document.createElement('button');
	clearButton.classList.add('btn', 'btn-default', 'btn-sm', 'clear');
	clearButton.innerHTML = '<i class="fa fa-ban"></i>';
	clearButton.addEventListener('click', event =>
		_onClearButtonClick(event, tableOfContents, manualContents, manualContentsScrollbar)
	);

	const searchBox = document.createElement('form');
	searchBox.classList.add('search-box', 'form-inline');
	searchBox.appendChild(inputField);
	searchBox.appendChild(previousButton);
	searchBox.appendChild(nextButton);
	searchBox.appendChild(clearButton);

	// Append search box to table of contents (after the heading).
	const tocContainer = tableOfContents.querySelector('.toc-container');
	const toc = tocContainer.querySelector('.toc');
	tocContainer.insertBefore(searchBox, toc);
}
