/* --------------------------------------------------------------
 manual.js 2018-09-12
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2018 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

import '../../node_modules/bootstrap/dist/css/bootstrap.css';
import '../../node_modules/font-awesome/css/font-awesome.css';
import '../../node_modules/perfect-scrollbar/css/perfect-scrollbar.css';
import '../common/styles/main.scss';
import {
    registerErrorHandler,
    registerPolyfills,
    parseGetParameter
} from './components/Environment';
import { prepareLayout, bindLayoutEventListeners } from './components/Layout';
import { createScrollToTopButton } from './components/ScrollToTopButton';
import { createManualSearchBox } from './components/ManualSearchBox';
import PerfectScrollbar from 'perfect-scrollbar';
import { fadeIn } from './components/Effects';
import ManualDocument from './components/ManualDocument';
import Breakpoints from './components/Breakpoints';
import { setFooterYear } from '../common/scripts/footer';
import { getApplicableVersion } from '../common/scripts/versions';
import { isElementInView } from '../common/scripts/view';

prepareLayout();
bindLayoutEventListeners();
registerErrorHandler();
registerPolyfills();
setFooterYear();

/**
 * Table Of Contents Width
 *
 * @type {Number}
 */
const tableOfContentsWidth = 300;

/**
 * Scrollbar Breakpoint
 *
 * Defines the width where the perfect-scrollbars are being used.
 *
 * @type {Number}
 */
const scrollbarBreakpoint = Breakpoints.xs + tableOfContentsWidth;

const tableOfContents = document.querySelector('.table-of-contents');
let tableOfContentsScrollbar;
const manualContents = document.querySelector('.manual-contents');
let manualContentsScrollbar;

let queryData = {};

/**
 * Get Manual HTML
 *
 * This method will fetch the manual HTML with an AJAX request.
 *
 * @return {Promise} Returns a promise object which will be resolved once the request is finished.
 *
 * @private
 */
const _getManualDocument = manualBaseUrl => {
    return new Promise((resolve, reject) => {
        const request = new XMLHttpRequest();
        request.open('GET', `${manualBaseUrl}/manual.html`, true);

        request.onload = function() {
            if (this.status >= 200 && this.status < 400) {
                resolve(this.response);
            } else {
                reject(this.response);
            }
        };

        request.onerror = function() {
            reject(this.response);
        };

        request.send();
    });
};

/**
 * Bind link click event listener.
 *
 * This method will bypass the table-of-contents links and will enable internal link navigation without
 * interference from page.js router.
 *
 * @return {Manual} Returns class instance for chained calls.
 *
 * @private
 */
const _bindLinkClickListener = () => {
    const tableOfContents = document.querySelector('.table-of-contents');
    const manualContents = document.querySelector('.manual-contents');

    tableOfContents.addEventListener('click', event => {
        event.preventDefault();

        if (event.target.tagName !== 'A') {
            return;
        }

        const target = document.getElementById(event.target.getAttribute('href').slice(1));

        if (window.innerWidth >= scrollbarBreakpoint) {
            manualContents.scrollTop = target.offsetTop;
            manualContentsScrollbar.update();
        } else {
            const scrollTop = target.getBoundingClientRect().top + document.body.scrollTop;
            window.scrollTo(0, scrollTop);
        }

        const url =
            location.hash.replace(/\?section=.*/, '') + `?section=${target.getAttribute('id')}`;
        history.pushState({ path: url }, '', url);
    });
};

/**
 * Bind window resize event listener.
 *
 * Make sure that the scrollbars are displayed correctly if the window is bigger than the "xs" breakpoint.
 *
 * @return {Manual} Returns class instance for chained calls.
 *
 * @private
 */
const _bindWindowResizeListener = () => {
    window.addEventListener('resize', () => _toggleContentScrollbars());
};

/**
 * Bind scroll spy event listener.
 *
 * This event listener will make sure that the currently displayed section will get the ".active" class in the
 * table of contents so that the user notices directly in which section he is.
 *
 * @private
 */
const _bindScrollSpyListener = manualContents => {
    manualContents.addEventListener('ps-scroll-y', () => {
        const sectionChangeStep =
            document.querySelector('header').offsetHeight +
            document.querySelector('.section1-title').offsetHeight;
        const currentScrollTop =
            document.querySelector('.manual-contents').scrollTop + sectionChangeStep;
        const topics = document.querySelectorAll('.topic');

        for (let i = topics.length - 1; i >= 0; i--) {
            const currentTopic = topics[i];

            if (!currentTopic.getBoundingClientRect || currentTopic.style.display === 'none') {
                continue;
            }

            if (currentTopic.offsetTop < currentScrollTop) {
                const href = `#${currentTopic.getAttribute('id')}`;
                const link = document.querySelector(`[href="${href}"]`);

                if (!link.classList.contains('active')) {
                    const currentLink = document.querySelector('.table-of-contents a.active');

                    link.classList.add('active');

                    if (currentLink) {
                        currentLink.classList.remove('active');
                    }
                }

                break;
            }
        }

        if (!document.querySelector('.table-of-contents .active')) {
            document.querySelector('.toc-entry-title').classList.add('active');
        }
    });

    // Add active class to first table of contents entry.
    const tocEntryTitle = document.querySelector('.toc-entry-title');

    if (tocEntryTitle && window.innerWidth > scrollbarBreakpoint) {
        tocEntryTitle.classList.add('active');
    }
};

/**
 * Toggle Content Scrollbars
 *
 * The scrollbars must always appear if the viewport is greater than the smallest breakpoint.
 *
 * @return {Manual} Returns class instance for chained calls.
 *
 * @private
 */
const _toggleContentScrollbars = () => {
    const tableOfContents = document.querySelector('.table-of-contents');
    const manualContents = document.querySelector('.manual-contents');

    if (window.innerWidth >= scrollbarBreakpoint && !tableOfContents.classList.contains('ps')) {
        const headerHeight = document.querySelector('header').offsetHeight;
        const footerHeight = document.querySelector('footer').offsetHeight;

        tableOfContents.style.height = manualContents.style.height = `calc(100vh - ${headerHeight +
            footerHeight}px)`;
        tableOfContentsScrollbar = new PerfectScrollbar(tableOfContents);
        manualContentsScrollbar = new PerfectScrollbar(manualContents, {
            minScrollbarLength: 50,
            wheelPropagation: true
        });
    }

    if (window.innerWidth < scrollbarBreakpoint && tableOfContents.classList.contains('ps')) {
        tableOfContents.style.height = manualContents.style.height = 'auto';

        const activeLink = tableOfContents.querySelector('a.active');

        if (activeLink) {
            activeLink.classList.remove('active');
        }

        tableOfContentsScrollbar.destroy();
        manualContentsScrollbar.destroy();
    }
};

/**
 * Display a message.
 *
 * @param {String} message The message to be displayed.
 *
 * @param {String} type Message type eg. info | warning. Default: info
 */
const _display = (message, type = 'info') => {
    const manualContents = document.querySelector('#manual');
    const div = document.createElement('div');

    div.innerHTML = `
        <div class="${type}">
            ${message}
            <i class="fa fa-times pull-right remove" aria-hidden="true"></i>
        </div>
    `;

    manualContents.prepend(div);

    const removeButton = document.querySelector('i.remove');
    removeButton.addEventListener('click', () => div.remove());
};

/**
 * Builds an Object from a GET Method's querystring
 *
 * @param queryString A query string to parse.
 *
 * @return {Object} Returns an Object with the querystring transformed into an object
 *
 * @private
 */
const _buildQueryObject = queryString => {
    // translates the queryString delimiters into JSON
    let queryJsonString =
        '"' +
        queryString
            .replace(/&/g, '",\n"') // e.g. section=foo&standalone=bar -> section=foo,standalone=bar
            .replace(/=/g, '":"') + // e.g. section=foo,standalone=bar -> section:foo,standalone:bar
        '"';
    // for non-value holding parameters, such as ?standalone&... add a value as boolean true
    queryJsonString = queryJsonString
        .split(',')
        .map(kv => {
            if (!kv.includes('":')) {
                // there is no value assigned to this key
                return kv + ':true';
            } else {
                // the key value pair is fine
                return kv;
            }
        })
        .join(',');

    // remove quotes around boolean values
    queryJsonString = queryJsonString.replace('"true"', 'true').replace('"false"', 'false');

    return JSON.parse('{' + queryJsonString + '}');
};

queryData = _buildQueryObject(location.search.slice(1));

const providedVersion = parseGetParameter('v');

getApplicableVersion(providedVersion).then(applicableVersion => {
    const manualBaseUrl = applicableVersion ? `docs/${applicableVersion}/manual` : `docs/manual`;

    _getManualDocument(manualBaseUrl)
        .then(response => {
            let manualDocument;

            // if the view is specified to be a standalone section and the section arg is not falsey
            if (queryData.standalone === true && !!queryData.section) {
                // extract the specific section from the response
                const parser = new DOMParser();
                const xmlDoc = parser.parseFromString(response, 'text/html');
                const sectionHtml = xmlDoc.getElementById(queryData.section).outerHTML;
                manualDocument = new ManualDocument(sectionHtml, applicableVersion, true);
                _toggleContentScrollbars();
                createScrollToTopButton(manualContents, manualContentsScrollbar);
            } else {
                manualDocument = new ManualDocument(response, applicableVersion);
                tableOfContents.innerHTML = manualDocument.getTableOfContents();

                fadeIn(tableOfContents);
                fadeIn(manualContents);

                tableOfContents.style.opacity = manualContents.style.opacity = 0;

                _toggleContentScrollbars();
                createScrollToTopButton(manualContents, manualContentsScrollbar);
                createManualSearchBox(tableOfContents, manualContents, manualContentsScrollbar);
                _bindLinkClickListener();
                _bindWindowResizeListener();
                _bindScrollSpyListener(manualContents);
            }

            if (providedVersion && providedVersion !== applicableVersion) {
                _display(
                    `Es wird Ihnen ein alternatives gültiges Handbuch (v${applicableVersion}) für Ihre Shop-Version ${providedVersion} angezeigt`
                );
            }

            manualContents.innerHTML = manualDocument.getManualContents();

			if ((queryData.section || location.hash) && !queryData.standalone) {
				const section = queryData.section || location.hash.replace(/#/, '');
				if (window.innerWidth >= scrollbarBreakpoint) {
					manualContents.scrollTop = document.getElementById(section).offsetTop;
					manualContentsScrollbar.update();
				} else {
					const scrollTop =
						document.getElementById(section).getBoundingClientRect().top +
						document.body.scrollTop;
					window.scrollTo(0, scrollTop);
				}
			}

            // Execute the search if provided by the GET parameter
            const search = parseGetParameter('search');
            search && _executeSearch(search);

            lazyLoadImages();
        })
        .catch(error => {
            _display('Keine Inhalt gefunden.', 'warning');
            console.error('Get Manual Document Error: ', error);
        });
});

/**
 * Executes the search event.
 *
 * @param keyword Keyword to be searched for.
 */
const _executeSearch = keyword => {
    const searchBox = document.querySelector('form.search-box input[type="text"]');
    searchBox.value = keyword;

    const enterKeyPressEvent = new KeyboardEvent('keyup', {
        bubbles: true,
        cancelable: true,
        keyCode: 13
    });
    searchBox.dispatchEvent(enterKeyPressEvent);
};

/**
 * Lazy loads all images which contain a css class of 'lazy'
 */
const lazyLoadImages = () => {
    let lazyImages = [].slice.call(document.querySelectorAll('img.lazy'));

    lazyImages.forEach(lazyImage => {
        if (isElementInView(lazyImage, 1000) && getComputedStyle(lazyImage).display !== 'none') {
            lazyImage.src = lazyImage.dataset.src;
            lazyImage.classList.remove('lazy');

            lazyImages = lazyImages.filter(function(image) {
                return image !== lazyImage;
            });

            if (lazyImages.length === 0) {
                manualContent.removeEventListener('ps-scroll-y', lazyLoadImages);
                manualContent.removeEventListener('scroll', lazyLoadImages);
                manualContent.removeEventListener('resize', lazyLoadImages);
                manualContent.removeEventListener('orientationchange', lazyLoadImages);
            }
        }
    });

    const manualContent = document.querySelector('.manual-contents');
    manualContent.addEventListener('ps-scroll-y', lazyLoadImages);
    manualContent.addEventListener('scroll', lazyLoadImages);
    manualContent.addEventListener('resize', lazyLoadImages);
    manualContent.addEventListener('orientationchange', lazyLoadImages);
};
