/* --------------------------------------------------------------
 tutorials.js 2018-09-12
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2018 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

import renderTutorials from './templates/tutorials.pug';
import marked from 'marked';
import hljs from 'highlightjs';
import '../../node_modules/highlightjs/styles/default.css';
import '../common/styles/main.scss';
import { setFooterYear } from '../common/scripts/footer';

$(function() {
	'use strict';

	$('#app').html(renderTutorials());
	setFooterYear();

	const $tutorials = $('.tutorials');
	const $content = $tutorials.find('.content div');
	const $toc = $tutorials.find('.toc ul');

	let urlVersion, urlPage;

	// Parse URL parameters (version and page).
	if (location.search !== '') {
		let parameters = location.search.substring(1).split('&');

		$.each(parameters, (index, parameter) => {
			if (parameter.indexOf('v=') > -1) {
				urlVersion = parameter.substring(2);
			} else if (parameter.indexOf('p=') > -1) {
				urlPage = decodeURIComponent(parameter.substring(2));
			}
		});
	}

	/**
	 * Whenever a result could not be found (error 404).
	 */
	function displayNoResultWarning() {
		$content.html('<div class="warning">Keine Ergebnis gefunden ...</div>').fadeIn('fast');
	}

	/**
	 * Sanitize the page title (no spaces, lowercase).
	 *
	 * @param {string} title
	 * @return {string}
	 */
	function sanitizeTitle(title) {
		return title.replace(/ /g, '-').toLowerCase();
	}

	/**
	 * Prepare Table Of Contents (Recursive)
	 *
	 * @param {object} page
	 * @param {jQuery} $parent
	 */
	function prepareTableOfContents(page, $parent) {
		let version = urlVersion ? 'v=' + urlVersion + '&' : '',
			href = '?' + version + 'p=' + sanitizeTitle(page.title),
			$li = $(`<li><a href="${href}">${page.title}</a></li>`);

		if (typeof page.file === 'string') {
			let version = urlVersion ? urlVersion + '/' : '';
			$li.find('a').data('file', `docs/${version}tutorials/${page.file}`);
		}

		if (page.pages !== undefined && page.pages.length > 0) {
			const $childList = $('<ul />');
			$li.append($childList);
			$.each(page.pages, (index, page) => {
				prepareTableOfContents(page, $childList);
			});
		}

		$parent.append($li);
	}

	/**
	 * Render Markdown File
	 *
	 * @param {string} file
	 */
	function renderMarkdown(file) {
		$.get(file)
			.done(md => {
				// Correct the relative links.
				const baseUrl = 'docs/' + (urlVersion ? urlVersion + '/' : '') + 'tutorials';
				md = md.replace(/\.\.\//g, baseUrl + '/');

				// Convert markdown into HTML and display it on the page.
				$content.html(marked(md));
				$content.find('pre code').each((index, block) => {
					hljs.highlightBlock(block);
				});
				$content.fadeIn('fast', () => {
					if (location.hash !== '') {
						$('a[href="' + location.hash + '"]').click(); // Navigate to sub-sections of the page.
					}
				});

				// Correct internal links to other markdown elements.
				$content.find('a').each((index, contentLink) => {
					if (
						$(contentLink).attr('href') &&
						$(contentLink)
							.attr('href')
							.indexOf('.md') > -1
					) {
						$toc.find('a').each((index, tocLink) => {
							if (
								$(tocLink)
									.data('file')
									.indexOf($(contentLink).attr('href')) > -1
							) {
								$(contentLink)
									.attr('href', $(tocLink).attr('href'))
									.data('file', $(tocLink).data('file'));
							}
						});
					}
				});
			})
			.fail(jqxhr => {
				if (jqxhr.status === 404) {
					displayNoResultWarning(jqxhr);
				}
			});
	}

	// Fetch and parse the tutorials.json file.
	const url = (urlVersion ? 'docs/' + urlVersion : 'docs') + '/tutorials/tutorials.json';

	$.getJSON(url)
		.done(page => {
			// Prepare table of contents.
			$.each(page.pages, (index, page) => {
				prepareTableOfContents(page, $toc);
			});
			$tutorials
				.find('.toc')
				.fadeTo(400, 1)
				.css('height', 'auto');

			// Table of contents click handling.
			$toc.on('click', 'li a', e => {
				const $link = $(e.target);

				if ($link.hasClass('active')) {
					e.preventDefault();
					return;
				}
			});

			// Render initial markdown file.
			let file;

			if (urlPage) {
				$toc.find('a').each((index, link) => {
					if (sanitizeTitle($(link).text()) === urlPage) {
						file = $(link).data('file');
						$(link).addClass('active');
						return false;
					}
				});
			} else if (typeof page.file === 'string') {
				file = (urlVersion ? 'docs/' + urlVersion : 'docs') + '/tutorials/' + page.file;
			}

			if (file) {
				renderMarkdown(file);
			}
		})
		.fail(jqxhr => {
			if (jqxhr.status === 404) {
				displayNoResultWarning();
				$tutorials.find('.toc').hide();
				$content.fadeTo(200, 1); // Change the initial opacity and height (fixes footer flickering).
			}
		})
		.always(() => {
			// Change the initial opacity and height (fixes footer flickering).
			$tutorials
				.find('.content')
				.fadeTo(400, 1)
				.css('height', 'auto');
		});
});
